@@ -1,4 +1,3 @@
-/* vi: set sw=4 ts=4: */
 /*
  * stat() for uClibc
  *
@@ -21,6 +20,14 @@ int stat(const char *file_name, struct stat *buf)
 	return fstatat(AT_FDCWD, file_name, buf, 0);
 }
 
+#elif __WORDSIZE == 64 && defined __NR_newfstatat && !defined __ARCH_HAS_DEPRECATED_SYSCALLS__
+# include <fcntl.h>
+
+int stat(const char *file_name, struct stat *buf)
+{
+	return fstatat64(AT_FDCWD, file_name, buf, 0);
+}
+
 #else
 # include "xstatconv.h"
 
