@@ -1,4 +1,3 @@
-/* vi: set sw=4 ts=4: */
 /*
  * A small little ldd implementation for uClibc
  *
@@ -15,6 +14,11 @@
 
 #include "porting.h"
 
+#if defined(__aarch64__)
+#define MATCH_MACHINE(x) (x == EM_AARCH64)
+#define ELFCLASSM	ELFCLASS64
+#endif
+
 #if defined(__alpha__)
 #define MATCH_MACHINE(x) (x == EM_ALPHA)
 #define ELFCLASSM	ELFCLASS64
@@ -50,6 +54,11 @@
 #define ELFCLASSM	ELFCLASS32
 #endif
 
+#if defined(__csky__)
+#define MATCH_MACHINE(x) (x == EM_MCORE)
+#define ELFCLASSM	ELFCLASS32
+#endif
+
 #if defined(__frv__)
 #define MATCH_MACHINE(x) (x == EM_CYGNUS_FRV)
 #define ELFCLASSM	ELFCLASS32
